@@ -86,7 +86,7 @@ char *keywords[] = {
 	[Mailaddr] = "mailaddr",
 	[Mailfrom] = "mailfrom",
 	[Program]  = "program",
-	[CreateDev]   = "create",
+	[CreateDev]= "create",
 	[Homehost] = "homehost",
 	[LTEnd]    = NULL
 };
@@ -255,6 +255,7 @@ mddev_dev_t load_partitions(void)
 }
 
 struct createinfo createinfo = {
+	.autof = 2, /* by default, create devices with standard names */
 	.symlinks = 1,
 #ifdef DEBIAN
 	.gid = 6, /* disk */
@@ -464,7 +465,7 @@ void arrayline(char *line)
 				fprintf(stderr, Name ": only specify bitmap file once. %s ignored\n",
 					w);
 			else
-				mis.bitmap_file = w+7;
+				mis.bitmap_file = strdup(w+7);
 
 		} else if (strncasecmp(w, "devices=", 8 ) == 0 ) {
 			if (mis.devices)
@@ -746,6 +747,22 @@ mddev_dev_t conf_get_devs()
 	return dlist;
 }
 
+int conf_test_dev(char *devname)
+{
+	struct conf_dev *cd;
+	if (cdevlist == NULL)
+		/* allow anything by default */
+		return 1;
+	for (cd = cdevlist ; cd ; cd = cd->next) {
+		if (strcasecmp(cd->name, "partitions") == 0)
+			return 1;
+		if (fnmatch(cd->name, devname, FNM_PATHNAME) == 0)
+			return 1;
+	}
+	return 0;
+}
+
+
 int match_oneof(char *devices, char *devname)
 {
     /* check if one of the comma separated patterns in devices
