@@ -170,7 +170,7 @@ void remove_partitions(int fd)
 #endif
 }
 
-int enough(int level, int raid_disks, int layout,
+int enough(int level, int raid_disks, int layout, int clean,
 	   char *avail, int avail_disks)
 {
 	int copies, first;
@@ -205,9 +205,15 @@ int enough(int level, int raid_disks, int layout,
 		return avail_disks >= 1;
 	case 4:
 	case 5:
-		return avail_disks >= raid_disks-1;
+		if (clean)
+			return avail_disks >= raid_disks-1;
+		else
+			return avail_disks >= raid_disks;
 	case 6:
-		return avail_disks >= raid_disks-2;
+		if (clean)
+			return avail_disks >= raid_disks-2;
+		else
+			return avail_disks >= raid_disks;
 	default:
 		return 0;
 	}
@@ -240,6 +246,25 @@ int same_uuid(int a[4], int b[4], int swapuuid)
 		return 0;
 	}
 }
+void copy_uuid(void *a, int b[4], int swapuuid)
+{
+	if (swapuuid) {
+		/* parse uuids are hostendian.
+		 * uuid's from some superblocks are big-ending
+		 * if there is a difference, we need to swap..
+		 */
+		unsigned char *ac = (unsigned char *)a;
+		unsigned char *bc = (unsigned char *)b;
+		int i;
+		for (i=0; i<16; i+= 4) {
+			ac[i+0] = bc[i+3];
+			ac[i+1] = bc[i+2];
+			ac[i+2] = bc[i+1];
+			ac[i+3] = bc[i+0];
+		}
+	} else
+		memcpy(a, b, 16);
+}
 
 #ifndef MDASSEMBLE
 int check_ext2(int fd, char *name)
@@ -767,7 +792,37 @@ struct supertype *guess_super(int fd)
 	return NULL;
 }
 
+/* Return size of device in bytes */
+int get_dev_size(int fd, char *dname, unsigned long long *sizep)
+{
+	unsigned long long ldsize;
+#ifdef BLKGETSIZE64
+	if (ioctl(fd, BLKGETSIZE64, &ldsize) != 0)
+#endif
+	{
+		unsigned long dsize;
+		if (ioctl(fd, BLKGETSIZE, &dsize) == 0) {
+			ldsize = dsize;
+			ldsize <<= 9;
+		} else {
+			if (dname)
+				fprintf(stderr, Name ": Cannot get size of %s: %s\b",
+					dname, strerror(errno));
+			return 0;
+		}
+	}
+	*sizep = ldsize;
+	return 1;
+}
 
+void get_one_disk(int mdfd, mdu_array_info_t *ainf, mdu_disk_info_t *disk)
+{
+	int d;
+	ioctl(mdfd, GET_ARRAY_INFO, ainf);
+	for (d = 0 ; d < ainf->raid_disks + ainf->nr_disks ; d++)
+		if (ioctl(mdfd, GET_DISK_INFO, disk) == 0)
+			return;
+}
 #ifdef __TINYC__
 /* tinyc doesn't optimize this check in ioctl.h out ... */
 unsigned int __invalid_size_argument_for_IOC = 0;
